<?php
/**
 * @file
 * Web style backend functionality.
 */

/**
 * Interface for a style backend of a site profile.
 */
interface ActonProfileBackendInterface {
  /**
   * Constructs a resource URL.
   *
   * Whether this method returns a valid URL or NULL depends on the behavior of
   * the actual implementation, e.g. if the constructor already has all the
   * information needed to reference a resource.
   *
   * @param string $path
   *   Resource path relative to the style server base.
   * @return mixed
   *   Absolute resource URL, or NULL if invalid parameters.
   */
  public function getResourceUrl($path);

  /**
   * Retrieves a style part.
   *
   * @param string $part
   *   Style part name.
   * @return mixed
   *   String style part content, or NULL if named part does not exist.
   */
  public function getStylePart($part);
}

/**
 * Interface for a backend providing site metadata.
 */
interface ActonProfileBackendInfoInterface {
  /**
   * Returns the site metadata.
   *
   * @return ActonProfileInfo|NULL
   *   Profile site info, or NULL if invalid site ID.
   */
  public function getSiteInfo();
}

/**
 * Default style backend.
 *
 * The default style backend uses the JSON format to interact with the style
 * server endpoint.
 */
class ActonProfileBackend extends ActonProfileInternalCache implements
  ActonProfileBackendInterface,
  ActonProfileBackendInfoInterface {
  /**
   * Style server endpoint.
   * @var ActonProfileEndpointInterface
   */
  protected $endpoint;

  /**
   * Style site ID.
   * @var int
   */
  protected $id;

  /**
   * Backend style server base URL.
   * @var string
   */
  protected $baseUrl;

  /**
   * Creates a backend.
   *
   * @param ActonProfileEndpointInterface $endpoint
   *   Endpoint for backend operations.
   * @param int $id
   *   Web style site ID.
   * @param string $baseUrl
   *   Style server base URL, for constructing resource URLs.
   */
  public function __construct(ActonProfileEndpointInterface $endpoint, $id, $baseUrl) {
    $this->endpoint = $endpoint;
    $this->id = $id;
    $this->baseUrl = $baseUrl;
  }

  /**
   * Returns the endpoint used by this backend.
   *
   * @return ActonProfileEndpointInterface
   */
  public function getEndpoint() {
    return $this->endpoint;
  }

  /**
   * {@inheritdoc}
   */
  public function getSiteInfo() {
    $keys = array('info', $this->id);
    if (!$info = $this->getInternalCache($keys)) {
      $json = $this->getEndpoint()->fetch($this->id, 'json');
      if (is_array($metadata = drupal_json_decode($json))) {
        $info = new ActonProfileInfo($metadata);
      }
      $this->setInternalCache($keys, $info);
    }
    return $info;
  }

  /**
   * {@inheritdoc}
   */
  public function getResourceUrl($path) {
    $protocol = !empty($GLOBALS['is_https']) ? 'https' : 'http';
    if ($info = $this->getSiteInfo()) {
      $path = $info->replaceVariables($path);
    }
    return "$protocol://{$this->baseUrl}/$path";
  }

  /**
   * {@inheritdoc}
   */
  public function getStylePart($part) {
    if ($info = $this->getSiteInfo()) {
      if (isset($info[$part]) && is_string($content = $info[$part])) {
        return $content;
      }
    }

    return NULL;
  }
}

/**
 * Site metadata container.
 */
class ActonProfileInfo extends ArrayObject {
  /**
   * Variable placeholder pattern.
   */
  const VARIABLE_PATTERN = '/\$\{([^}]+)\}/';

  /**
   * Creates a profile info container for given metadata.
   *
   * In order for a backend site profile to be valid, the site status variable
   * $this['sitevars']['status'] must be non-NULL.
   */
  public function __construct(array $data) {
    parent::__construct($data);
    $this->setUp($data);
  }

  /**
   * Returns whether this profile info is valid.
   *
   * @return bool
   *   TRUE if the supplied site ID is valid.
   */
  public function isValid() {
    return isset($this['sitevars']) && isset($this['sitevars']['status']);
  }

  /**
   * Sets up the container, interpreting metadata as necessary.
   */
  protected function setUp(array $data) {
    // Set up defaults.
    if (!isset($this['sitevars'])) {
      $this['sitevars'] = array();
    }

    // Interpret variables.
    $this->setUpVersion($data);
  }

  /**
   * Sets up the version.
   */
  protected function setUpVersion(array $data) {
    // Check the default style metadata property, and then start guessing.
    foreach (array('newstyle', 'ver', 'version') as $key) {
      if (!empty($data['sitevars'][$key])) {
        $this['style_version'] = (int) $data['sitevars'][$key];
        break;
      }
    }
  }

  /**
   * Replaces placeholders in a string with profile info variables.
   *
   * All placeholders in the form of ${VARIABLE} are replaced. All variables set
   * up in this container are available for replacement. Boolean values
   * true/false are respectively treated as 1/0. See {@see
   * ActonProfileInfo::selectVariable()} for the format of the selector.
   *
   * @see selectVariable()
   */
  public function replaceVariables($text) {
    return preg_replace_callback(self::VARIABLE_PATTERN, array($this, 'pregReplaceCallback'), $text);
  }

  /**
   * {@inheritdoc}
   */
  public function offsetGet($index) {
    // Gag E_NOTICE.
    if (!$this->offsetExists($index)) {
      return NULL;
    }

    return parent::offsetGet($index);
  }

  /**
   * Replaces a matched variable placeholder for preg_replace_callback().
   */
  public function pregReplaceCallback($matches) {
    $value = $this->selectVariable($matches[1]);
    if (is_bool($value)) {
      return $value ? 1 : 0;
    }
    else {
      return $value;
    }
  }

  /**
   * Selects a nested variable using a colon-delimited selector.
   *
   * The selector uses a colon to reference a nested variable inside an array.
   * For example, 'sitevars:site_id' references the site ID in the web style
   * site variables.
   */
  public function selectVariable($selector) {
    $value = $this;
    $keys = explode(':', $selector);
    foreach ($keys as $key) {
      if (isset($value[$key])) {
        $value = $value[$key];
      }
      else {
        return NULL;
      }
    }
    return $value;
  }
}

/**
 * Interface for a remote style server endpoint.
 */
interface ActonProfileEndpointInterface {
  /**
   * Determines if this endpoint is valid.
   *
   * @return bool
   */
  public function isValid();

  /**
   * Fetches a part from the style server.
   *
   * @param int $id
   *   Site ID on the style server.
   * @param string $part
   *   Name of style part.
   * @param array $params
   *   Extra URL parameters.
   * @return mixed
   *   Style part content.
   */
  public function fetch($id, $part, array $params = array());

  /**
   * Returns a default URL parameter.
   *
   * @param string $key
   *   URL parameter name.
   * @return mixed
   *   URL parameter value, or NULL is not set.
   */
  public function getParam($key);

  /**
   * Returns all parameters.
   *
   * @return array
   *   All default URL parameters.
   */
  public function getParams();

  /**
   * Sets a default URL parameter.
   *
   * @param string $key
   *   URL parameter name.
   * @param mixed $value
   *   URL parameter value. If NULL is given and the parameter has been set, the
   *   named parameter is removed.
   */
  public function setParam($key, $value);
}

/**
 * Style server endpoint.
 */
class ActonProfileEndpoint implements ActonProfileEndpointInterface {
  /**
   * Style server endpoint URL.
   */
  protected $url;

  /**
   * Backend options.
   * @var array
   */
  protected $params;

  /**
   * Whether the endpoint is valid.
   */
  private $valid;

  /**
   * Creates an instance from fulL URL.
   */
  public static function fromUrl($url, array $params = array()) {
    return new ActonProfileEndpoint($url, $params);
  }

  /**
   * Creates an instance from separate base and endpoint URLs.
   */
  public static function fromBaseSuffix($base, $suffix, array $params = array()) {
    return new ActonProfileEndpoint("http://{$base}{$suffix}", $params);
  }

  /**
   * Constructs a style server backend.
   *
   * @param string $url
   *   The style server endpoint URL.
   * @param array $params
   *   URL parameters.
   */
  public function __construct($url, array $params = array()) {
    $this->url = $url;
    $this->params = $params;
  }

  /**
   * {@inheritdoc}
   */
  public function getParam($key) {
    return isset($this->params[$key]) ? $this->params[$key] : NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function setParam($key, $value) {
    if (isset($value)) {
      $this->params[$key] = $value;
    }
    elseif (array_key_exists($key, $this->params)) {
      unset($this->params[$key]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getParams() {
    return $this->params;
  }

  /**
   * {@inheritdoc}
   */
  public function isValid() {
    if (!isset($this->valid)) {
      $result = drupal_http_request($this->getUrl(array(), FALSE));
      $this->valid = !isset($result->error);
    }
    return $this->valid;
  }

  /**
   * {@inheritdoc}
   */
  public function fetch($id, $part, array $params = array()) {
    $url = $this->getUrl(compact('id', 'part') + $params);
    $result = drupal_http_request($url);
    if (!isset($result->error) && isset($result->data)) {
      return $result->data;
    }
    return NULL;
  }

  /**
   * Returns the full endpoint request URL.
   */
  public function getUrl(array $params = array(), $defaults = TRUE) {
    if ($defaults) {
      $params += $this->params;
    }
    $query = empty($params) ? '' : ('?' . drupal_http_build_query($params));
    return $this->url . $query;
  }
}

/**
 * Base decorator for an endpoint.
 */
abstract class ActonProfileEndpointDecorator implements ActonProfileEndpointInterface {
  /**
   * Decorated endpoint.
   * @var ActonProfileEndpointInterface
   */
  protected $decoratedEndpoint;

  /**
   * Creates the decorator for an endpoint.
   */
  public function __construct(ActonProfileEndpointInterface $endpoint) {
    $this->decoratedEndpoint = $endpoint;
  }

  /**
   * {@inheritdoc}
   */
  public function isValid() {
    return $this->decoratedEndpoint->isValid();
  }

  /**
   * {@inheritdoc}
   */
  public function fetch($id, $part, array $params = array()) {
    return $this->decoratedEndpoint->fetch($id, $part, $params);
  }

  /**
   * {@inheritdoc}
   */
  public function getParam($key) {
    return $this->decoratedEndpoint->getParam($key);
  }

  /**
   * {@inheritdoc}
   */
  public function getParams() {
    return $this->decoratedEndpoint->getParams();
  }

  /**
   * {@inheritdoc}
   */
  public function setParam($key, $value) {
    $this->decoratedEndpoint->setParam($key, $value);
  }

  /**
   * Returns the decorated endpoint.
   */
  public function getDecoratedEndpoint() {
    return $this->decoratedEndpoint;
  }
}

/**
 * Cached fetch endpoint decorator.
 */
class ActonProfileCachedEndpoint extends ActonProfileEndpointDecorator {
  /**
   * Bin name for caching fetched results.
   * @var string
   */
  protected $cacheBin;

  /**
   * Cache lifetime.
   * @var int
   */
  protected $cacheLifetime;

  /**
   * Creates a cache decorator for an endpoint.
   */
  public function __construct(ActonProfileEndpointInterface $endpoint, $cacheBin = 'cache', $cacheLifetime = CACHE_PERMANENT) {
    parent::__construct($endpoint);
    $this->cacheBin = $cacheBin;
    $this->cacheLifetime = $cacheLifetime;
  }

  /**
   * {@inheritdoc}
   *
   * Fetched results are cached by parameters.
   */
  public function fetch($id, $part, array $params = array()) {
    $parts = &drupal_static(__METHOD__, array());

    $key = $this->getCacheKey($id, $part, $params + $this->getParams());
    if (!isset($parts[$key])) {
      if ($cache = cache_get($key, $this->cacheBin)) {
        $parts[$key] = $cache->data;
      }
      else {
        $parts[$key] = $this->decoratedEndpoint->fetch($id, $part, $params);
        cache_set($key, $parts[$key], $this->cacheBin, $this->getCacheExpire());
      }
    }

    return $parts[$key];
  }

  /**
   * Computes a cache key from an array of parameters.
   */
  protected function getCacheKey($id, $part, array $params) {
    $query = http_build_query(compact('id', 'part') + $params);
    return 'acton_profile:endpoint:' . md5($query);
  }

  /**
   * Computes the cache expiration timestamp.
   */
  protected function getCacheExpire() {
    return ($this->cacheLifetime == CACHE_PERMANENT)
      ? CACHE_PERMANENT
      : (REQUEST_TIME + $this->cacheLifetime);
  }
}
