<?php
/**
 * @file
 * Theme hooks and callbacks to be included with module.
 */

/**
 * Implements hook_preprocess_html().
 */
function acton_profile_preprocess_html(&$variables) {
  if ($profile = acton_profile_get_active_profile()) {
    $variables['acton_profile'] = $profile;
    $controller = acton_profile_style_controller();

    // Set up for specific capabilities.
    if ($controller instanceof ActonProfileStyleCapabilitiesInterface && $capabilities = $controller->getCapabilities($profile)) {
      // Set up for HTML5.
      if ($capabilities->hasCapability(ActonProfileCapabilities::HTML5)) {
        $variables['doctype'] = '<!DOCTYPE html>';
        $variables['encoding'] = array(
          '#tag' => 'meta',
          '#attributes' => array(
            'charset' => 'UTF-8',
          ),
        );

        // Add RDF prefixes.
        if (module_exists('rdf')) {
          $rdfa_prefixes = array();
          foreach (rdf_get_namespaces() as $prefix => $uri) {
            $rdfa_prefixes[] = $prefix . ': ' . $uri;
          }
          if ($rdfa_prefixes) {
            $variables['html_extra_attributes_array']['prefix'] = implode(' ', $rdfa_prefixes);
          }
        }
      }

      // Remove GRDDL profile attribute.
      if (!$capabilities->hasCapability(ActonProfileCapabilities::GRDDL)) {
        $variables['grddl_profile'] = '';
      }

      // Set up for responsive design.
      if ($capabilities->hasCapability(ActonProfileCapabilities::RESPONSIVE)) {
        // Specify viewport for responsive design.
        $viewport = array(
          '#tag' => 'meta',
          '#attributes' => array(
            'name' => 'viewport',
            'content' => 'width=device-width, initial-scale=1',
          ),
        );
        drupal_add_html_head($viewport, 'viewport');

        // Enable responsive table implementation.
        drupal_add_js(drupal_get_path('theme', 'acton') . '/js/responsive-tables.js');
      }
    }

    // Attach style scripts.
    _acton_profile_style_add_js($profile);

    // Fall back to XHTML.
    if (empty($variables['doctype'])) {
      // Switch the doctype depending on whether RDF is enabled.
      $variables['doctype'] = module_exists('rdf')
        ? '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML+RDFa 1.1//EN" "http://www.w3.org/MarkUp/DTD/xhtml-rdfa-2.dtd">'
        : '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">';

      // Add XHTML attributes.
      $variables['html_attributes_array']['xmlns'] = 'http://www.w3.org/1999/xhtml';
      $variables['html_attributes_array']['xml:lang'] = $variables['language']->language;
      if (module_exists('rdf')) {
        foreach (rdf_get_namespaces() as $prefix => $uri) {
          $variables['html_extra_attributes_array']['xmlns:' . $prefix] = $uri;
        }
      }
    }

    // Set up generic HTML attributes.
    $variables['html_attributes_array']['lang'] = $variables['language']->language;
    $variables['html_attributes_array']['dir'] = $variables['language']->dir;

    // Generate browser title.
    if ($profile->data['site_short_name']) {
      $variables['head_title_array']['name'] = check_plain($profile->data['site_short_name']);
    }
    if (isset($variables['head_title_array']['name']) && $variables['head_title_array']['name'] != 'ANU') {
      $variables['head_title_array']['org'] = 'ANU';
    }
    $variables['head_title'] = implode(' - ', $variables['head_title_array']);

    // Add page parts.
    $variables['anu_head'] = array(
      '#theme' => 'acton_anu_head',
      '#profile' => $profile,
    );
    if (!empty($profile->settings['metadata_use'])) {
      $metadata = acton_profile_get_active_metadata('meta_description', 'meta_subject', 'date_modified', 'creator', 'creator_address');
      $variables['acton_head'] = array(
        '#theme' => 'acton_head',
        '#identifier' => check_plain(url(current_path(), array('absolute' => TRUE))),
        '#title' => drupal_get_title(),
        '#description' => $metadata['meta_description'],
        '#subject' => $metadata['meta_subject'],
        '#date_modified' => !empty($metadata['date_modified']) ? date('c', $metadata['date_modified']) : '',
        '#creator' => $metadata['creator'],
        '#creator_address' => $metadata['creator_address'],
      );
    }
  }
}

/**
 * Attaches script resources for styles.
 */
function _acton_profile_style_add_js($profile) {
  $controller = acton_profile_style_controller();
  if ($controller instanceof ActonProfileStyleBackendInterface && $backend = $controller->getBackend($profile)) {
    $scripts_base = $backend->getResourceUrl('${style_version}/scripts');

    // Add centrally hosted jQuery plugins.
    drupal_add_js("$scripts_base/jquery.hoverIntent.js", array(
      'type' => 'external',
      'group' => JS_LIBRARY,
      'weight' => 5,
    ));

    // Alias jQuery library.
    drupal_add_js('var $anujq = jQuery.noConflict();', array(
      'type' => 'inline',
      'group' => JS_LIBRARY,
      'weight' => 5,
    ));

    // Add style scripts.
    drupal_add_js("$scripts_base/anu-common.js", array(
      'type' => 'external',
      'weight' => 5,
    ));

    if ($controller instanceof ActonProfileStyleCapabilitiesInterface && $capabilities = $controller->getCapabilities($profile)) {
      if ($capabilities->hasCapability(ActonProfileCapabilities::MODERNIZR)) {
        $filename = $capabilities->getCapabilityParameter(ActonProfileCapabilities::MODERNIZR, ActonProfileCapabilities::PARAMETER_LIBRARY_FILENAME);
        drupal_add_js("$scripts_base/$filename", array(
          'type' => 'external',
          'weight' => 5,
        ));
      }
      if ($capabilities->hasCapability(ActonProfileCapabilities::RESPONSIVE)) {
        drupal_add_js("$scripts_base/anu-menu.js", array(
          'type' => 'external',
          'weight' => 5,
        ));
      }
      if ($capabilities->hasCapability(ActonProfileCapabilities::MEGAMENU)) {
        drupal_add_js("$scripts_base/anu-mega-menu.js", array(
          'type' => 'external',
          'weight' => 5,
        ));
      }
    }
  }
}

/**
 * Implements hook_preprocess_page().
 */
function acton_profile_preprocess_page(&$variables) {
  // Give the Acton theme the currently active profile.
  $profile = acton_profile_get_active_profile();
  if (!$profile) {
    acton_profile_set_alert_message(t('Please configure a !default_profile.', array('!default_profile' => l(t('default profile'), 'admin/appearance/acton-profile'))));
  }
  else {
    $variables['acton_profile'] = $profile;

    $variables['anu_body_header'] = array(
      '#theme' => 'acton_anu_body_header',
      '#profile' => $profile,
    );
    $controller = acton_profile_style_controller();
    if ($controller instanceof ActonProfileStyleCapabilitiesInterface && $capabilities = $controller->getCapabilities($profile)) {
      $main_menu_source = variable_get('menu_main_links_source', 'main-menu');
      $utility_menu_source = variable_get('menu_secondary_links_source', 'user-menu');

      // Add utility menu.
      if ($capabilities->hasCapability(ActonProfileCapabilities::UTILITY_MENU)) {
        if ($placeholder_name = $capabilities->getCapabilityParameter(ActonProfileCapabilities::PARAMETER_PLACEHOLDER_NAME, ActonProfileCapabilities::UTILITY_MENU)) {
          $variables['anu_body_header']['#utility_placeholder'] = "<!--$placeholder_name-->";
        }
        if (menu_load($utility_menu_source)) {
          $variables['anu_body_header']['#utility'] = array(
            '#theme' => 'acton_utility_menu',
            '#menu' => $utility_menu_source,
          );
        }
      }

      // Add horizontal tabs.
      if ($capabilities->hasCapability(ActonProfileCapabilities::BANNER_TABS)) {
        if ($placeholder_name = $capabilities->getCapabilityParameter(ActonProfileCapabilities::PARAMETER_PLACEHOLDER_NAME, ActonProfileCapabilities::BANNER_TABS)) {
          $variables['anu_body_header']['#tabs_placeholder'] = "<!--$placeholder_name-->";
        }
        $variables['anu_body_header']['#tabs'] = array(
          '#theme' => 'acton_banner_tabs',
          '#links' => $variables['main_menu'],
        );
      }

      // Add megamenu.
      if ($capabilities->hasCapability(ActonProfileCapabilities::MEGAMENU)) {
        if ($placeholder_name = $capabilities->getCapabilityParameter(ActonProfileCapabilities::PARAMETER_PLACEHOLDER_NAME, ActonProfileCapabilities::MEGAMENU)) {
          $variables['anu_body_header']['#mega_placeholder'] = "<!--$placeholder_name-->";
        }
        if (menu_load($main_menu_source)) {
          $variables['anu_body_header']['#mega'] = array(
            '#theme' => 'acton_hnav_mega',
            '#profile' => $profile,
            '#menu' => $main_menu_source,
          );
        }

        if (menu_load($utility_menu_source)) {
          $variables['anu_body_header']['#mega']['#utility_menu'] = $utility_menu_source;
        }
      }
    }
    $variables['anu_body_footer'] = array(
      '#theme' => 'acton_anu_body_footer',
      '#profile' => $profile,
    );

    // Prepare site footer.
    $metadata = acton_profile_get_active_metadata('resp_officer', 'resp_officer_address', 'site_contact', 'site_contact_address', 'date_modified', 'about_page');
    $variables['site_footer'] = array(
      '#theme' => 'acton_site_footer',
      '#resp_officer' => $metadata['resp_officer'],
      '#resp_officer_address' => $metadata['resp_officer_address'],
      '#site_contact' => $metadata['site_contact'],
      '#site_contact_address' => $metadata['site_contact_address'],
      '#date_modified' => !empty($metadata['date_modified']) ? date('j F Y', $metadata['date_modified']) : '',
      '#about_page' => !empty($metadata['about_page']) ? url($metadata['about_page']) : '',
    );
  }
}

/**
 * Implements hook_process_page().
 */
function acton_profile_process_page(&$variables) {
  if ($alert_bar_message = acton_profile_get_formatted_alert_message()) {
    $variables['alert_bar'] = array(
      '#theme' => 'acton_alert_bar',
      '#message' => $alert_bar_message,
    );
  }
}

/**
 * Implements hook_preprocess_region().
 */
function acton_profile_preprocess_region(&$variables) {
  if ($variables['region'] == 'sidebar_first' && $profile = acton_profile_get_active_profile()) {
    $variables['acton_profile'] = $profile;

    // Add search box in left sidebar.
    $search_box_mini = array();
    if (!empty($profile->settings['search_show']) && !empty($profile->settings['search_site_name'])) {
      $search_site_name = $profile->settings['search_site_name'];
      $variables['search_box'] = array(
        '#theme' => 'acton_search_box',
        '#site_short_name' => $profile->data['site_short_name'],
        '#search_site_name' => $search_site_name,
      );
      $search_box_mini = array(
        '#theme' => 'acton_search_box_mini',
        '#search_site_name' => $search_site_name,
      );
    }

    // Add responsive menu.
    $controller = acton_profile_style_controller();
    if ($controller instanceof ActonProfileStyleCapabilitiesInterface && $capabilities = $controller->getCapabilities($profile)) {
      if ($capabilities->hasCapability(ActonProfileCapabilities::RESPONSIVE)) {
        // Prepare theme variable.
        if ($controller instanceof ActonProfileStyleBackendInterface) {
          $backend = $controller->getBackend($profile);
          if ($backend instanceof ActonProfileBackendInterface) {
            $menu_icon_src = $backend->getResourceUrl('${style_version}/images/buttons/btn-menu.gif');
            $variables['responsive_menu'] = array(
              '#theme' => 'acton_responsive_menu',
              '#menu_icon_src' => $menu_icon_src,
              '#search_box_mini' => $search_box_mini,
            );
          }
        }
      }
    }
  }
}

/**
 * Clean string variables.
 */
function _acton_clean_variables(&$variables, array $keys) {
  foreach ($keys as $key) {
    if (is_scalar($key) && $variables[$key]) {
      $variables[$key] = check_plain($variables[$key]);
    }
  }
}

/**
 * Implements hook_preprocess_link().
 */
function acton_profile_preprocess_link(&$variables) {
  if (!empty($variables['options']['acton_banner_tab']) && !empty($variables['options']['attributes']['class'])) {
    $class = &$variables['options']['attributes']['class'];
    if (in_array('active', $class) || in_array('active-trail', $class)) {
      $class[] = 'tabs-select';
    }
  }
}
