<?php
/**
 * @file
 * Style controller interfaces and implementations.
 */

/**
 * Controller interface for providing style backends.
 */
interface ActonProfileStyleBackendInterface {
  /**
   * Returns an instance of backend for a profile.
   *
   * If a profile is not given, default variables are used where applicable.
   *
   * @param ActonProfile $profile
   *   Profile entity.
   * @return ActonProfileBackendInterface
   */
  public function getBackend(ActonProfile $profile = NULL);
}

/**
 * Controller interface for obtaining style info.
 */
interface ActonProfileStyleInfoInterface {
  /**
   * Returns the site info for a profile.
   *
   * @param ActonProfile $profile
   * @return ActonProfileInfo|NULL
   *   Site info, or NULL if invalid profile or backend.
   */
  public function getSiteInfo(ActonProfile $profile);
}

/**
 * Controller interface for determining site profile style capabilities.
 */
interface ActonProfileStyleCapabilitiesInterface {
  /**
   * Returns capabilities implementation for a profile.
   *
   * @param ActonProfile $profile
   * @return ActonProfileCapabilitiesInterface|NULL
   *   Profile style capabilities, or NULL if invalid profile or backend.
   */
  public function getCapabilities(ActonProfile $profile);
}

/**
 * Controller interface for providing style endpoints.
 */
interface ActonProfileStyleEndpointInterface {
  /**
   * Returns an instance of style endpoint for a profile.
   *
   * If a profile is not given, default variables are used where applicable.
   *
   * @param ActonProfile $profile
   *   Profile entity.
   * @return ActonProfileEndpointInterface
   */
  public function getEndpoint(ActonProfile $profile = NULL);
}

/**
 * Interface for basic style controller functionality.
 *
 * A controller implementing this interface triggers the default page rendering
 * workflow.
 */
interface ActonProfileStyleControllerInterface extends
  ActonProfileStyleBackendInterface,
  ActonProfileStyleInfoInterface,
  ActonProfileStyleCapabilitiesInterface,
  ActonProfileStyleEndpointInterface {}

/**
 * Controller interface for providing form handlers for style configuration.
 */
interface ActonProfileStyleFormInterface {
  /**
   * Returns an instance of profile form delegate.
   *
   * @return ActonProfileFormDelegateInterface
   */
  public function getFormDelegate();

  /**
   * Returns an instance of settings form controller.
   *
   * @return ActonProfileSettingsFormControllerInterface
   */
  public function getSettingsFormController();
}

/**
 * Base implementation for managing cached values.
 *
 * This base implementation provides instance caching functions by arbitrary
 * composite keys.
 */
abstract class ActonProfileInternalCache {
  /**
   * Cached instances.
   */
  private $cache = array();

  /**
   * Destroys cached instances.
   */
  public function __destruct() {
    unset($this->cache);
  }

  /**
   * Returns a cached value.
   *
   * @param array $keys
   *   A composite key. All keys must be scalar values or objects.
   * @return mixed
   *   The value referenced by the given keys, or NULL if invalid key or the
   *   referenced value is not found.
   */
  protected function getInternalCache(array $keys) {
    $cache = &$this->cache;
    if ($keys = $this->getNormalizedCacheKeys($keys)) {
      // Obtain cache reference.
      foreach ($keys as $key) {
        // Validate key.
        if (!isset($key)) {
          return NULL;
        }
        // Check cache.
        if (!isset($cache[$key])) {
          return NULL;
        }
        // Obtain cache reference.
        $cache = &$cache[$key];
      }
      // Return value.
      return $cache;
    }
    return NULL;
  }

  /**
   * Caches a value.
   *
   * @param array $keys
   *   A composite key. All keys must be scalar values or objects.
   * @param mixed $value
   */
  protected function setInternalCache(array $keys, $value) {
    $cache = &$this->cache;
    if ($keys = $this->getNormalizedCacheKeys($keys)) {
      // Validate keys.
      if (in_array(NULL, $keys, TRUE)) {
        return;
      }
      // Obtain cache reference.
      foreach ($keys as $key) {
        // Check cache.
        if (!isset($cache[$key])) {
          $cache[$key] = array();
        }
        // Obtain cache reference.
        $cache = &$cache[$key];
      }
      // Set value.
      $cache = $value;
    }
  }

  /**
   * Normalizes cache keys.
   */
  private function getNormalizedCacheKeys(array $keys) {
    $normalizedKeys = array();
    foreach ($keys as $key) {
      // Abort given a NULL key.
      // Transform object key.
      if (is_object($key)) {
        $key = spl_object_hash($key);
      }
      // Validate key.
      elseif (isset($key) && !is_scalar($key)) {
        return NULL;
      }
      // Push key.
      $normalizedKeys[] = $key;
    }
    return $normalizedKeys;
  }

  /**
   * Resets cached values.
   *
   * @param array $keys
   *   The composite key for which to reset cached values. All keys must be
   *   scalar values or objects. If empty, all cached values are reset.
   */
  protected function resetInternalCache(array $keys = array()) {
    // Reset all values.
    if (!$keys) {
      $this->cache = array();
    }
    // Reset keyed values.
    else {
      $this->setInternalCache($keys, NULL);
    }
  }
}

/**
 * Default style controller implementation.
 */
class ActonProfileDefaultStyleController extends ActonProfileInternalCache implements
  ActonProfileStyleControllerInterface,
  ActonProfileStyleFormInterface {
  /**
   * {@inheritdoc}
   */
  public function getBackend(ActonProfile $profile = NULL) {
    $keys = array('backend', $profile ? $profile : 'default');
    if (!$instance = $this->getInternalCache($keys)) {
      $this->setInternalCache($keys, $instance = $this->createBackend($profile));
    }
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getEndpoint(ActonProfile $profile = NULL) {
    $keys = array('endpoint', $profile ? $profile : 'default');
    if (!$instance = $this->getInternalCache($keys)) {
      $this->setInternalCache($keys, $instance = $this->createEndpoint($profile));
    }
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getSiteInfo(ActonProfile $profile) {
    $backend = $this->getBackend($profile);
    if ($backend instanceof ActonProfileBackendInfoInterface) {
      return $backend->getSiteInfo();
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getCapabilities(ActonProfile $profile) {
    if ($profile) {
      $keys = array('capabilities', $profile);
      if (!$instance = $this->getInternalCache($keys)) {
        $this->setInternalCache($keys, $instance = $this->createCapabilities($profile));
      }
      return $instance;
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormDelegate() {
    $keys = array('form_delegate');
    if (!$instance = $this->getInternalCache($keys)) {
      $this->setInternalCache($keys, $instance = $this->createFormDelegate());
    }
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getSettingsFormController() {
    $keys = array('settings_form_controller');
    if (!$instance = $this->getInternalCache($keys)) {
      $this->setInternalCache($keys, $instance = $this->createSettingsFormController());
    }
    return $instance;
  }

  /**
   * Creates a backend for a profile entity.
   */
  protected function createBackend(ActonProfile $profile = NULL) {
    list($base) = $this->getEndpointUrlParts($profile);
    $id = isset($profile->data['site_id']) ? $profile->data['site_id'] : '';
    return new ActonProfileBackend($this->getEndpoint($profile), $id, $base);
  }

  /**
   * Creates an endpoint for a profile entity.
   */
  protected function createEndpoint(ActonProfile $profile = NULL) {
    list($base, $suffix) = $this->getEndpointUrlParts($profile);
    $endpoint = ActonProfileEndpoint::fromBaseSuffix($base, $suffix);
    $endpoint = $this->prepareEndpoint($endpoint, $profile);
    return $endpoint;
  }

  /**
   * Prepares a created endpoint.
   */
  protected function prepareEndpoint(ActonProfileEndpoint $endpoint, ActonProfile $profile = NULL) {
    // Use cached results.
    if (variable_get('acton_profile_cache')) {
      $cacheBin = 'cache_acton_profile';
      $cacheLifetime = variable_get('acton_profile_cache_lifetime', ACTON_PROFILE_DEFAULT_CACHE_LIFE);
    }
    else {
      $cacheBin = 'cache';
      $cacheLifetime = CACHE_PERMANENT;
    }
    $endpoint = new ActonProfileCachedEndpoint($endpoint, $cacheBin, $cacheLifetime);

    // Set up endpoint parameters.
    if ($GLOBALS['is_https']) {
      $endpoint->setParam('ssl', 1);
    }
    $endpoint->setParam('appid', sprintf('Drupal %s', VERSION));
    if ($profile && !empty($profile->settings['style_version'])) {
      $endpoint->setParam('ver', $profile->settings['style_version']);
    }
    // Suppress jQuery library per Web Style.
    $endpoint->setParam('nojq', 2);
    // Obtain minified assets.
    $endpoint->setParam('minify', 1);

    return $endpoint;
  }

  /**
   * Creates a capabilities checker for a profile entity.
   */
  protected function createCapabilities(ActonProfile $profile) {
    if ($info = $this->getSiteInfo($profile)) {
      return new ActonProfileCapabilities($info);
    }
    return NULL;
  }

  /**
   * Creates a profile form delegate for a profile entity.
   */
  protected function createFormDelegate() {
    return new ActonProfileDefaultFormDelegate();
  }

  /**
   * Creates a settings form controller.
   */
  protected function createSettingsFormController() {
    return new ActonProfileDefaultSettingsFormController();
  }

  /**
   * Returns endpoint parts for a profile.
   */
  protected function getEndpointUrlParts(ActonProfile $profile = NULL) {
    $keys = array('url', $profile ? $profile : 'default');
    if (!$parts = $this->getInternalCache($keys)) {
      // Construct parts using profile.
      if ($profile) {
        $settings = $profile->settings + array(
          'server_override' => FALSE,
          'server_endpoint_base' => '',
          'server_endpoint_suffix' => '',
        );
        if ($settings['server_override']) {
          $base = $settings['server_endpoint_base'];
          $suffix = $settings['server_endpoint_suffix'];
        }
      }
      $base = !empty($base) ? $base : variable_get('acton_profile_default_endpoint_base', ACTON_PROFILE_DEFAULT_ENDPOINT_BASE);
      $suffix = !empty($suffix) ? $suffix : variable_get('acton_profile_default_endpoint_suffix', ACTON_PROFILE_DEFAULT_ENDPOINT_SUFFIX);
      $parts = array($base, $suffix);
      $this->setInternalCache($keys, $parts);
    }
    return $parts;
  }

  /**
   * Resets cached instances for profile.
   */
  public function resetProfileCache(ActonProfile $profile = NULL) {
    if (isset($profile)) {
      $this->resetInternalCache(array('backend', $profile));
      $this->resetInternalCache(array('endpoint', $profile));
    }
    else {
      $this->resetInternalCache(array('backend'));
      $this->resetInternalCache(array('endpoint'));
    }
  }

  /**
   * Resets cached form instances.
   */
  public function resetFormCache() {
    $this->resetInternalCache(array('form_delegate'));
    $this->resetInternalCache(array('settings_form_controller'));
  }
}

/**
 * Interface for site profile style capability detection.
 */
interface ActonProfileCapabilitiesInterface {
  /**
   * Checks whether the site supports a named capability.
   *
   * @param mixed $capability
   *   Capability identifier. The specific capabilities implementation will
   *   decide what capabilities it can interpret.
   * @return bool
   *   TRUE if capable, FALSE otherwise. A capability that cannot be interpreted
   *   should correspond to FALSE.
   */
  public function hasCapability($capability);

  /**
   * Returns parameter value for given capability in the site profile.
   *
   * This method does not specifically check for whether a capability is
   * supported, so it is up to the invocation to perform a sanity check.
   *
   * @param mixed $capability
   *   Capability identifier.
   * @param mixed $parameter
   *   Parameter name.
   * @return mixed
   *   Parameter value.
   */
  public function getCapabilityParameter($capability, $parameter);
}

/**
 * Default capabilities implementation.
 */
class ActonProfileCapabilities implements ActonProfileCapabilitiesInterface {
  /**
   * HTML5 capability.
   */
  const HTML5 = 'default.html5';

  /**
   * GRDDL profile.
   */
  const GRDDL = 'default.grddl';

  /**
   * Responsive styles.
   */
  const RESPONSIVE = 'default.responsive';

  /**
   * Modernizr library support in styles.
   */
  const MODERNIZR = 'default.modernizr';

  /**
   * Horizontal tab banner.
   */
  const UTILITY_MENU = 'default.utility_menu';

  /**
   * Horizontal tab banner.
   */
  const BANNER_TABS = 'default.tabs';

  /**
   * Horizontal tab banner.
   */
  const MEGAMENU = 'default.megamenu';

  /**
   * Parameter for (versioned) library file name.
   */
  const PARAMETER_LIBRARY_FILENAME = 'default.library.filename';

  /**
   * Parameter for placeholder string.
   */
  const PARAMETER_PLACEHOLDER_NAME = 'default.placeholder';

  /**
   * Profile style info.
   * @var ActonProfileInfo
   */
  protected $info;

  /**
   * Style version.
   */
  private $version;

  /**
   * Creates an instance of a capability checker.
   */
  public function __construct(ActonProfileInfo $info) {
    $this->info = $info;
    $this->version = !empty($info['style_version']) ? $info['style_version'] : 0;
  }

  /**
   * {@inheritdoc}
   */
  public function hasCapability($capability) {
    switch ($capability) {
      // Support features from before version 4.
      case self::GRDDL:
        return $this->version < 4;

      // Support features since version 4.
      case self::HTML5:
      case self::RESPONSIVE:
        return $this->version >= 4;

      // Support features specific to version 4.
      case self::MODERNIZR:
        return $this->version >= 4;

      case self::UTILITY_MENU:
        return (bool) $this->getSiteVar('utility_menu_str');

      case self::MEGAMENU:
        return $this->getSiteVar('hnav_type') == 'mega-other';

      case self::BANNER_TABS:
        return $this->getSiteVar('hnav_type') == 'tabs';

      default:
        return FALSE;
    }
  }

  /**
   * Returns remote site profile variable.
   */
  protected function getSiteVar($name) {
    return isset($this->info['sitevars'][$name]) ? $this->info['sitevars'][$name] : NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getCapabilityParameter($capability, $parameter) {
    switch ($parameter) {
      // Determine library file name.
      case self::PARAMETER_LIBRARY_FILENAME:
        switch ($capability) {
          case self::MODERNIZR:
            return 'modernizr-2.0.6.min.js';
        }
        break;

      // Navigation placeholders.
      case self::PARAMETER_PLACEHOLDER_NAME:
        switch ($capability) {
          case self::UTILITY_MENU:
            return $this->getSiteVar('utility_menu_str');

          case self::MEGAMENU:
          case self::BANNER_TABS:
            return $this->getSiteVar('hnav_str');
        }
        break;
    }

    return NULL;
  }
}
